#![feature(int_roundings, ip)]

use std::sync::Arc;

use tables::{interface::InterfaceTable, neighbor::NeighborTable};
use tokio::sync::Mutex;
use tracing_subscriber::{prelude::*, util::SubscriberInitExt};

mod config;
mod nht;
mod tables;
mod threads;
// mod timer;
mod tlvs;
mod transport;

#[tokio::main(flavor = "current_thread")]
async fn main() -> anyhow::Result<()> {
    tracing_subscriber::registry()
        .with(tracing_subscriber::fmt::layer())
        .with(tracing_subscriber::EnvFilter::from_env("BABELD_LOG"))
        .init();

    let conf = config::Conf::load(
        std::env::var("BABELD_CONFIG").unwrap_or_else(|_| "./babeld.toml".to_string()),
    )
    .await?;
    let (config, advertise) = conf.get()?;

    let mut interfaces = InterfaceTable::new();
    let neighbors = Arc::new(Mutex::new(NeighborTable::new()));

    for (interface, iface_config) in &config.interfaces {
        tracing::info!("Activating on interface {interface}");
        interfaces
            .add_by_name(
                &interface,
                *iface_config,
                advertise.clone(),
                neighbors.clone(),
            )
            .await?;
    }

    let int = interfaces
        .get_interface(nht::Nht::get_ifindex_by_name(
            config.interfaces.keys().next().unwrap(),
        )?)
        .unwrap();
    int.worker.handle(neighbors).await?;

    Ok(())
}
