use std::net::{IpAddr, SocketAddr};

pub struct Nht {}

impl Nht {
    pub fn get_ifname_by_index(index: u32) -> anyhow::Result<String> {
        let mut ifname_str = String::new();
        let ifname =
            unsafe { libc::if_indextoname(index, std::ptr::addr_of_mut!(ifname_str) as *mut i8) };

        let ifname = unsafe { std::ffi::CStr::from_ptr(ifname) }
            .to_string_lossy()
            .to_string();

        Ok(ifname)
    }

    pub fn get_ifindex_by_name(ifname: &str) -> anyhow::Result<u32> {
        let ifindex = nix::net::if_::if_nametoindex(ifname)?;
        Ok(ifindex)
    }

    pub fn get_ifaddrs(ifname: String) -> anyhow::Result<Vec<cidr::IpInet>> {
        let mut ret = Vec::new();

        let ifaddrs = nix::ifaddrs::getifaddrs()?;
        for ifaddr in ifaddrs {
            if ifaddr.interface_name == ifname {
                let Some(addr) = ifaddr.address else { continue; };
                let ipaddr: Result<SocketAddr, _> = addr.to_string().parse();
                if ipaddr.is_err() {
                    continue;
                }
                ret.push(cidr::IpInet::new_host(ipaddr.unwrap().ip()));
            }
        }

        Ok(ret)
    }

    pub fn get_primary_v4(addresses: Vec<cidr::IpInet>) -> Option<IpAddr> {
        for addr in addresses {
            if addr.is_ipv4() {
                return Some(addr.address());
            }
        }
        None
    }
}
